function sRGB = processColor(bayerDATA, AsShotNeutral, ColorMatrix1, CalibrationIlluminant1)

% function sRGB = processColor(bayerDATA, AsShotNeutral, ColorMatrix1, CalibrationIlluminant1)
%
% convert bayer ordered data to rawRGB array,
% apply color correction resulting from AsShotNeutral (white balance),
% ColorMatrix1 and CalibrationIlluminant1
% -> return sRGB ready to display (except for gamma correction)
%
% If AsShotNeutral, ColorMatrix1 or CalibrationIlluminant1
% are not given or empty arrays they will be set to defaults

% set defaults for arguments not given
if nargin<2, AsShotNeutral = []; end
if length(AsShotNeutral) == 0 % not given
  AsShotNeutral = [0.8702 1.000 0.6961]; % define defaults
end

if nargin<3, ColorMatrix1 = []; end
if length(ColorMatrix1) == 0 % not given in file
  ColorMatrix1 = [ % define defaults
     1.771600  -0.540400  -0.167400
    -0.284500   1.249400   0.024700
    -0.230000   0.623600   0.647100
  ];
end

if nargin<4, CalibrationIlluminant1 = []; end
if length(CalibrationIlluminant1) == 0
  CalibrationIlluminant1 = 20; % assume we have D55 light source
end

% additional color conversion, as sRGB output to display assumes D65
% see link for details
% http://www.brucelindbloom.com/index.html?Eqn_ChromAdapt.html
% using Bradford solution (as Photoshop does)
switch CalibrationIlluminant1
  case 20 % CalibrationIlluminant1 = 20 (D55)
    XYZD55_to_XYZD65 = [ 
       0.9726856 -0.0135482  0.0361731
      -0.0167463  1.0049102  0.0120598
       0.0070026 -0.0116372  1.1869548
    ];
  case 21 % CalibrationIlluminant1 = 21 (D65)
    XYZD65_to_XYZD65 = [
     1 0 0
     0 1 0
     0 0 1
    ];
  case 22 % CalibrationIlluminant1 = 22 (D75)
    XYZD75_to_XYZD65 = [
       1.0206905  0.0091588 -0.0228796
       0.0115005  0.9984917 -0.0076762
      -0.0043619  0.0072053  0.8853432
    ];
  case 23 % CalibrationIlluminant1 = 23 (D50)
    XYZD50_to_XYZD65 = [
       0.9555766 -0.0230393  0.0631636
      -0.0282895  1.0099416  0.0210077
       0.0122982 -0.0204830  1.3299098
    ];
end

rawRGB = bayer2rgb(bayerDATA); % convert bayer pattern to rgb
      
% calculate final color matrix
ColorMatrix = diag(1 ./ AsShotNeutral); % sensor sensitivity / lighting correction
ColorMatrix *= ColorMatrix1; % ColorMatrix from file or default
ColorMatrix *= XYZD55_to_XYZD65; % output should be D65 illuminant
%ColorMatrix *= XYZD65_to_sRGB; % not required
sRGB = zeros(size(rawRGB));
for n=1:3
  sRGB(:,:,n) = ...
      ColorMatrix(n,1)*rawRGB(:,:,1) ...
    + ColorMatrix(n,2)*rawRGB(:,:,2) ...
    + ColorMatrix(n,3)*rawRGB(:,:,3);
end
sRGB(find(sRGB<0)) = 0; % clip negative values